/****************************************************************************
** Meta object code from reading C++ file 'qwaylandivisurface.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/extensions/qwaylandivisurface.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandivisurface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandIviSurface_t {
    QByteArrayData data[17];
    char stringdata0[221];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandIviSurface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandIviSurface_t qt_meta_stringdata_QWaylandIviSurface = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QWaylandIviSurface"
QT_MOC_LITERAL(1, 19, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 35, 4), // "data"
QT_MOC_LITERAL(3, 40, 14), // "surfaceChanged"
QT_MOC_LITERAL(4, 55, 0), // ""
QT_MOC_LITERAL(5, 56, 12), // "iviIdChanged"
QT_MOC_LITERAL(6, 69, 10), // "initialize"
QT_MOC_LITERAL(7, 80, 23), // "QWaylandIviApplication*"
QT_MOC_LITERAL(8, 104, 14), // "iviApplication"
QT_MOC_LITERAL(9, 119, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(10, 136, 7), // "surface"
QT_MOC_LITERAL(11, 144, 5), // "iviId"
QT_MOC_LITERAL(12, 150, 16), // "QWaylandResource"
QT_MOC_LITERAL(13, 167, 8), // "resource"
QT_MOC_LITERAL(14, 176, 13), // "sendConfigure"
QT_MOC_LITERAL(15, 190, 4), // "size"
QT_MOC_LITERAL(16, 195, 25) // "QQmlListProperty<QObject>"

    },
    "QWaylandIviSurface\0DefaultProperty\0"
    "data\0surfaceChanged\0\0iviIdChanged\0"
    "initialize\0QWaylandIviApplication*\0"
    "iviApplication\0QWaylandSurface*\0surface\0"
    "iviId\0QWaylandResource\0resource\0"
    "sendConfigure\0size\0QQmlListProperty<QObject>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandIviSurface[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       4,   16, // methods
       3,   50, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   36,    4, 0x06 /* Public */,
       5,    0,   37,    4, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       6,    4,   38,    4, 0x02 /* Public */,
      14,    1,   47,    4, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 9, QMetaType::UInt, 0x80000000 | 12,    8,   10,   11,   13,
    QMetaType::Void, QMetaType::QSize,   15,

 // properties: name, type, flags
       2, 0x80000000 | 16, 0x00094009,
      10, 0x80000000 | 9, 0x00495009,
      11, QMetaType::UInt, 0x00495001,

 // properties: notify_signal_id
       0,
       0,
       1,

       0        // eod
};

void QWaylandIviSurface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandIviSurface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->surfaceChanged(); break;
        case 1: _t->iviIdChanged(); break;
        case 2: _t->initialize((*reinterpret_cast< QWaylandIviApplication*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3])),(*reinterpret_cast< const QWaylandResource(*)>(_a[4]))); break;
        case 3: _t->sendConfigure((*reinterpret_cast< const QSize(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandIviSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandIviSurface::surfaceChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandIviSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandIviSurface::iviIdChanged)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandIviSurface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->data(); break;
        case 1: *reinterpret_cast< QWaylandSurface**>(_v) = _t->surface(); break;
        case 2: *reinterpret_cast< uint*>(_v) = _t->iviId(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandIviSurface::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandShellSurfaceTemplate<QWaylandIviSurface>::staticMetaObject>(),
    qt_meta_stringdata_QWaylandIviSurface.data,
    qt_meta_data_QWaylandIviSurface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandIviSurface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandIviSurface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandIviSurface.stringdata0))
        return static_cast<void*>(this);
    return QWaylandShellSurfaceTemplate<QWaylandIviSurface>::qt_metacast(_clname);
}

int QWaylandIviSurface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandShellSurfaceTemplate<QWaylandIviSurface>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandIviSurface::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandIviSurface::iviIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
