/****************************************************************************
** Meta object code from reading C++ file 'qwaylandxdgoutputv1.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/extensions/qwaylandxdgoutputv1.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandxdgoutputv1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandXdgOutputManagerV1_t {
    QByteArrayData data[1];
    char stringdata0[27];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandXdgOutputManagerV1_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandXdgOutputManagerV1_t qt_meta_stringdata_QWaylandXdgOutputManagerV1 = {
    {
QT_MOC_LITERAL(0, 0, 26) // "QWaylandXdgOutputManagerV1"

    },
    "QWaylandXdgOutputManagerV1"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandXdgOutputManagerV1[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QWaylandXdgOutputManagerV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QWaylandXdgOutputManagerV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandCompositorExtensionTemplate<QWaylandXdgOutputManagerV1>::staticMetaObject>(),
    qt_meta_stringdata_QWaylandXdgOutputManagerV1.data,
    qt_meta_data_QWaylandXdgOutputManagerV1,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandXdgOutputManagerV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandXdgOutputManagerV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandXdgOutputManagerV1.stringdata0))
        return static_cast<void*>(this);
    return QWaylandCompositorExtensionTemplate<QWaylandXdgOutputManagerV1>::qt_metacast(_clname);
}

int QWaylandXdgOutputManagerV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandCompositorExtensionTemplate<QWaylandXdgOutputManagerV1>::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QWaylandXdgOutputV1_t {
    QByteArrayData data[21];
    char stringdata0[313];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandXdgOutputV1_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandXdgOutputV1_t qt_meta_stringdata_QWaylandXdgOutputV1 = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QWaylandXdgOutputV1"
QT_MOC_LITERAL(1, 20, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 36, 4), // "data"
QT_MOC_LITERAL(3, 41, 14), // "managerChanged"
QT_MOC_LITERAL(4, 56, 0), // ""
QT_MOC_LITERAL(5, 57, 13), // "outputChanged"
QT_MOC_LITERAL(6, 71, 22), // "logicalPositionChanged"
QT_MOC_LITERAL(7, 94, 18), // "logicalSizeChanged"
QT_MOC_LITERAL(8, 113, 22), // "logicalGeometryChanged"
QT_MOC_LITERAL(9, 136, 11), // "nameChanged"
QT_MOC_LITERAL(10, 148, 18), // "descriptionChanged"
QT_MOC_LITERAL(11, 167, 25), // "QQmlListProperty<QObject>"
QT_MOC_LITERAL(12, 193, 7), // "manager"
QT_MOC_LITERAL(13, 201, 27), // "QWaylandXdgOutputManagerV1*"
QT_MOC_LITERAL(14, 229, 6), // "output"
QT_MOC_LITERAL(15, 236, 15), // "QWaylandOutput*"
QT_MOC_LITERAL(16, 252, 4), // "name"
QT_MOC_LITERAL(17, 257, 11), // "description"
QT_MOC_LITERAL(18, 269, 15), // "logicalPosition"
QT_MOC_LITERAL(19, 285, 11), // "logicalSize"
QT_MOC_LITERAL(20, 297, 15) // "logicalGeometry"

    },
    "QWaylandXdgOutputV1\0DefaultProperty\0"
    "data\0managerChanged\0\0outputChanged\0"
    "logicalPositionChanged\0logicalSizeChanged\0"
    "logicalGeometryChanged\0nameChanged\0"
    "descriptionChanged\0QQmlListProperty<QObject>\0"
    "manager\0QWaylandXdgOutputManagerV1*\0"
    "output\0QWaylandOutput*\0name\0description\0"
    "logicalPosition\0logicalSize\0logicalGeometry"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandXdgOutputV1[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       7,   16, // methods
       8,   58, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   51,    4, 0x06 /* Public */,
       5,    0,   52,    4, 0x06 /* Public */,
       6,    0,   53,    4, 0x06 /* Public */,
       7,    0,   54,    4, 0x06 /* Public */,
       8,    0,   55,    4, 0x06 /* Public */,
       9,    0,   56,    4, 0x06 /* Public */,
      10,    0,   57,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       2, 0x80000000 | 11, 0x00094009,
      12, 0x80000000 | 13, 0x00495009,
      14, 0x80000000 | 15, 0x00495009,
      16, QMetaType::QString, 0x00495103,
      17, QMetaType::QString, 0x00495103,
      18, QMetaType::QPoint, 0x00495103,
      19, QMetaType::QSize, 0x00495103,
      20, QMetaType::QRect, 0x00495001,

 // properties: notify_signal_id
       0,
       0,
       1,
       5,
       6,
       2,
       3,
       4,

       0        // eod
};

void QWaylandXdgOutputV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandXdgOutputV1 *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->managerChanged(); break;
        case 1: _t->outputChanged(); break;
        case 2: _t->logicalPositionChanged(); break;
        case 3: _t->logicalSizeChanged(); break;
        case 4: _t->logicalGeometryChanged(); break;
        case 5: _t->nameChanged(); break;
        case 6: _t->descriptionChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandXdgOutputV1::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgOutputV1::managerChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgOutputV1::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgOutputV1::outputChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgOutputV1::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgOutputV1::logicalPositionChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgOutputV1::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgOutputV1::logicalSizeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgOutputV1::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgOutputV1::logicalGeometryChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgOutputV1::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgOutputV1::nameChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgOutputV1::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgOutputV1::descriptionChanged)) {
                *result = 6;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgOutputManagerV1* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandXdgOutputV1 *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->data(); break;
        case 1: *reinterpret_cast< QWaylandXdgOutputManagerV1**>(_v) = _t->manager(); break;
        case 2: *reinterpret_cast< QWaylandOutput**>(_v) = _t->output(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->name(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->description(); break;
        case 5: *reinterpret_cast< QPoint*>(_v) = _t->logicalPosition(); break;
        case 6: *reinterpret_cast< QSize*>(_v) = _t->logicalSize(); break;
        case 7: *reinterpret_cast< QRect*>(_v) = _t->logicalGeometry(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QWaylandXdgOutputV1 *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 3: _t->setName(*reinterpret_cast< QString*>(_v)); break;
        case 4: _t->setDescription(*reinterpret_cast< QString*>(_v)); break;
        case 5: _t->setLogicalPosition(*reinterpret_cast< QPoint*>(_v)); break;
        case 6: _t->setLogicalSize(*reinterpret_cast< QSize*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandXdgOutputV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QWaylandXdgOutputV1.data,
    qt_meta_data_QWaylandXdgOutputV1,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandXdgOutputV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandXdgOutputV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandXdgOutputV1.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QWaylandXdgOutputV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandXdgOutputV1::managerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandXdgOutputV1::outputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QWaylandXdgOutputV1::logicalPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QWaylandXdgOutputV1::logicalSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QWaylandXdgOutputV1::logicalGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QWaylandXdgOutputV1::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QWaylandXdgOutputV1::descriptionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
