/****************************************************************************
** Meta object code from reading C++ file 'qwaylandxdgshellv6integration_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/extensions/qwaylandxdgshellv6integration_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandxdgshellv6integration_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtWayland__XdgToplevelV6Integration_t {
    QByteArrayData data[19];
    char stringdata0[362];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtWayland__XdgToplevelV6Integration_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtWayland__XdgToplevelV6Integration_t qt_meta_stringdata_QtWayland__XdgToplevelV6Integration = {
    {
QT_MOC_LITERAL(0, 0, 35), // "QtWayland::XdgToplevelV6Integ..."
QT_MOC_LITERAL(1, 36, 15), // "handleStartMove"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 13), // "QWaylandSeat*"
QT_MOC_LITERAL(4, 67, 4), // "seat"
QT_MOC_LITERAL(5, 72, 17), // "handleStartResize"
QT_MOC_LITERAL(6, 90, 9), // "Qt::Edges"
QT_MOC_LITERAL(7, 100, 5), // "edges"
QT_MOC_LITERAL(8, 106, 18), // "handleSetMaximized"
QT_MOC_LITERAL(9, 125, 20), // "handleUnsetMaximized"
QT_MOC_LITERAL(10, 146, 22), // "handleMaximizedChanged"
QT_MOC_LITERAL(11, 169, 19), // "handleSetFullscreen"
QT_MOC_LITERAL(12, 189, 21), // "handleUnsetFullscreen"
QT_MOC_LITERAL(13, 211, 23), // "handleFullscreenChanged"
QT_MOC_LITERAL(14, 235, 22), // "handleActivatedChanged"
QT_MOC_LITERAL(15, 258, 24), // "handleSurfaceSizeChanged"
QT_MOC_LITERAL(16, 283, 23), // "handleToplevelDestroyed"
QT_MOC_LITERAL(17, 307, 26), // "handleMaximizedSizeChanged"
QT_MOC_LITERAL(18, 334, 27) // "handleFullscreenSizeChanged"

    },
    "QtWayland::XdgToplevelV6Integration\0"
    "handleStartMove\0\0QWaylandSeat*\0seat\0"
    "handleStartResize\0Qt::Edges\0edges\0"
    "handleSetMaximized\0handleUnsetMaximized\0"
    "handleMaximizedChanged\0handleSetFullscreen\0"
    "handleUnsetFullscreen\0handleFullscreenChanged\0"
    "handleActivatedChanged\0handleSurfaceSizeChanged\0"
    "handleToplevelDestroyed\0"
    "handleMaximizedSizeChanged\0"
    "handleFullscreenSizeChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtWayland__XdgToplevelV6Integration[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x08 /* Private */,
       5,    2,   82,    2, 0x08 /* Private */,
       8,    0,   87,    2, 0x08 /* Private */,
       9,    0,   88,    2, 0x08 /* Private */,
      10,    0,   89,    2, 0x08 /* Private */,
      11,    0,   90,    2, 0x08 /* Private */,
      12,    0,   91,    2, 0x08 /* Private */,
      13,    0,   92,    2, 0x08 /* Private */,
      14,    0,   93,    2, 0x08 /* Private */,
      15,    0,   94,    2, 0x08 /* Private */,
      16,    0,   95,    2, 0x08 /* Private */,
      17,    0,   96,    2, 0x08 /* Private */,
      18,    0,   97,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 6,    4,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QtWayland::XdgToplevelV6Integration::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<XdgToplevelV6Integration *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleStartMove((*reinterpret_cast< QWaylandSeat*(*)>(_a[1]))); break;
        case 1: _t->handleStartResize((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< Qt::Edges(*)>(_a[2]))); break;
        case 2: _t->handleSetMaximized(); break;
        case 3: _t->handleUnsetMaximized(); break;
        case 4: _t->handleMaximizedChanged(); break;
        case 5: _t->handleSetFullscreen(); break;
        case 6: _t->handleUnsetFullscreen(); break;
        case 7: _t->handleFullscreenChanged(); break;
        case 8: _t->handleActivatedChanged(); break;
        case 9: _t->handleSurfaceSizeChanged(); break;
        case 10: _t->handleToplevelDestroyed(); break;
        case 11: _t->handleMaximizedSizeChanged(); break;
        case 12: _t->handleFullscreenSizeChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QtWayland::XdgToplevelV6Integration::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandQuickShellIntegration::staticMetaObject>(),
    qt_meta_stringdata_QtWayland__XdgToplevelV6Integration.data,
    qt_meta_data_QtWayland__XdgToplevelV6Integration,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtWayland::XdgToplevelV6Integration::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtWayland::XdgToplevelV6Integration::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtWayland__XdgToplevelV6Integration.stringdata0))
        return static_cast<void*>(this);
    return QWaylandQuickShellIntegration::qt_metacast(_clname);
}

int QtWayland::XdgToplevelV6Integration::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandQuickShellIntegration::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}
struct qt_meta_stringdata_QtWayland__XdgPopupV6Integration_t {
    QByteArrayData data[3];
    char stringdata0[56];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtWayland__XdgPopupV6Integration_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtWayland__XdgPopupV6Integration_t qt_meta_stringdata_QtWayland__XdgPopupV6Integration = {
    {
QT_MOC_LITERAL(0, 0, 32), // "QtWayland::XdgPopupV6Integration"
QT_MOC_LITERAL(1, 33, 21), // "handleGeometryChanged"
QT_MOC_LITERAL(2, 55, 0) // ""

    },
    "QtWayland::XdgPopupV6Integration\0"
    "handleGeometryChanged\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtWayland__XdgPopupV6Integration[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void QtWayland::XdgPopupV6Integration::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<XdgPopupV6Integration *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleGeometryChanged(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QtWayland::XdgPopupV6Integration::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandQuickShellIntegration::staticMetaObject>(),
    qt_meta_stringdata_QtWayland__XdgPopupV6Integration.data,
    qt_meta_data_QtWayland__XdgPopupV6Integration,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtWayland::XdgPopupV6Integration::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtWayland::XdgPopupV6Integration::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtWayland__XdgPopupV6Integration.stringdata0))
        return static_cast<void*>(this);
    return QWaylandQuickShellIntegration::qt_metacast(_clname);
}

int QtWayland::XdgPopupV6Integration::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandQuickShellIntegration::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
