/****************************************************************************
** Meta object code from reading C++ file 'qwaylandwlshellintegration_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/extensions/qwaylandwlshellintegration_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandwlshellintegration_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtWayland__WlShellIntegration_t {
    QByteArrayData data[30];
    char stringdata0[527];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtWayland__WlShellIntegration_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtWayland__WlShellIntegration_t qt_meta_stringdata_QtWayland__WlShellIntegration = {
    {
QT_MOC_LITERAL(0, 0, 29), // "QtWayland::WlShellIntegration"
QT_MOC_LITERAL(1, 30, 15), // "handleStartMove"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 13), // "QWaylandSeat*"
QT_MOC_LITERAL(4, 61, 4), // "seat"
QT_MOC_LITERAL(5, 66, 17), // "handleStartResize"
QT_MOC_LITERAL(6, 84, 34), // "QWaylandWlShellSurface::Resiz..."
QT_MOC_LITERAL(7, 119, 5), // "edges"
QT_MOC_LITERAL(8, 125, 24), // "handleSetDefaultTopLevel"
QT_MOC_LITERAL(9, 150, 18), // "handleSetTransient"
QT_MOC_LITERAL(10, 169, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(11, 186, 13), // "parentSurface"
QT_MOC_LITERAL(12, 200, 16), // "relativeToParent"
QT_MOC_LITERAL(13, 217, 8), // "inactive"
QT_MOC_LITERAL(14, 226, 18), // "handleSetMaximized"
QT_MOC_LITERAL(15, 245, 15), // "QWaylandOutput*"
QT_MOC_LITERAL(16, 261, 6), // "output"
QT_MOC_LITERAL(17, 268, 19), // "handleSetFullScreen"
QT_MOC_LITERAL(18, 288, 40), // "QWaylandWlShellSurface::FullS..."
QT_MOC_LITERAL(19, 329, 6), // "method"
QT_MOC_LITERAL(20, 336, 9), // "framerate"
QT_MOC_LITERAL(21, 346, 14), // "handleSetPopup"
QT_MOC_LITERAL(22, 361, 6), // "parent"
QT_MOC_LITERAL(23, 368, 27), // "handleShellSurfaceDestroyed"
QT_MOC_LITERAL(24, 396, 30), // "handleSurfaceHasContentChanged"
QT_MOC_LITERAL(25, 427, 12), // "handleRedraw"
QT_MOC_LITERAL(26, 440, 24), // "adjustOffsetForNextFrame"
QT_MOC_LITERAL(27, 465, 6), // "offset"
QT_MOC_LITERAL(28, 472, 27), // "handleFullScreenSizeChanged"
QT_MOC_LITERAL(29, 500, 26) // "handleMaximizedSizeChanged"

    },
    "QtWayland::WlShellIntegration\0"
    "handleStartMove\0\0QWaylandSeat*\0seat\0"
    "handleStartResize\0QWaylandWlShellSurface::ResizeEdge\0"
    "edges\0handleSetDefaultTopLevel\0"
    "handleSetTransient\0QWaylandSurface*\0"
    "parentSurface\0relativeToParent\0inactive\0"
    "handleSetMaximized\0QWaylandOutput*\0"
    "output\0handleSetFullScreen\0"
    "QWaylandWlShellSurface::FullScreenMethod\0"
    "method\0framerate\0handleSetPopup\0parent\0"
    "handleShellSurfaceDestroyed\0"
    "handleSurfaceHasContentChanged\0"
    "handleRedraw\0adjustOffsetForNextFrame\0"
    "offset\0handleFullScreenSizeChanged\0"
    "handleMaximizedSizeChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtWayland__WlShellIntegration[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x08 /* Private */,
       5,    2,   82,    2, 0x08 /* Private */,
       8,    0,   87,    2, 0x08 /* Private */,
       9,    3,   88,    2, 0x08 /* Private */,
      14,    1,   95,    2, 0x08 /* Private */,
      17,    3,   98,    2, 0x08 /* Private */,
      21,    3,  105,    2, 0x08 /* Private */,
      23,    0,  112,    2, 0x08 /* Private */,
      24,    0,  113,    2, 0x08 /* Private */,
      25,    0,  114,    2, 0x08 /* Private */,
      26,    1,  115,    2, 0x08 /* Private */,
      28,    0,  118,    2, 0x08 /* Private */,
      29,    0,  119,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 6,    4,    7,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10, QMetaType::QPoint, QMetaType::Bool,   11,   12,   13,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, 0x80000000 | 18, QMetaType::UInt, 0x80000000 | 15,   19,   20,   16,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 10, QMetaType::QPoint,    4,   22,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPointF,   27,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QtWayland::WlShellIntegration::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<WlShellIntegration *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleStartMove((*reinterpret_cast< QWaylandSeat*(*)>(_a[1]))); break;
        case 1: _t->handleStartResize((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< QWaylandWlShellSurface::ResizeEdge(*)>(_a[2]))); break;
        case 2: _t->handleSetDefaultTopLevel(); break;
        case 3: _t->handleSetTransient((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< const QPoint(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 4: _t->handleSetMaximized((*reinterpret_cast< QWaylandOutput*(*)>(_a[1]))); break;
        case 5: _t->handleSetFullScreen((*reinterpret_cast< QWaylandWlShellSurface::FullScreenMethod(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< QWaylandOutput*(*)>(_a[3]))); break;
        case 6: _t->handleSetPopup((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3]))); break;
        case 7: _t->handleShellSurfaceDestroyed(); break;
        case 8: _t->handleSurfaceHasContentChanged(); break;
        case 9: _t->handleRedraw(); break;
        case 10: _t->adjustOffsetForNextFrame((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 11: _t->handleFullScreenSizeChanged(); break;
        case 12: _t->handleMaximizedSizeChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandOutput* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandOutput* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QtWayland::WlShellIntegration::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandQuickShellIntegration::staticMetaObject>(),
    qt_meta_stringdata_QtWayland__WlShellIntegration.data,
    qt_meta_data_QtWayland__WlShellIntegration,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtWayland::WlShellIntegration::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtWayland::WlShellIntegration::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtWayland__WlShellIntegration.stringdata0))
        return static_cast<void*>(this);
    return QWaylandQuickShellIntegration::qt_metacast(_clname);
}

int QtWayland::WlShellIntegration::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandQuickShellIntegration::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
