/****************************************************************************
** Meta object code from reading C++ file 'qwaylandseat.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/compositor_api/qwaylandseat.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandseat.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandSeat_t {
    QByteArrayData data[34];
    char stringdata0[433];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandSeat_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandSeat_t qt_meta_stringdata_QWaylandSeat = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QWaylandSeat"
QT_MOC_LITERAL(1, 13, 17), // "mouseFocusChanged"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 13), // "QWaylandView*"
QT_MOC_LITERAL(4, 46, 8), // "newFocus"
QT_MOC_LITERAL(5, 55, 8), // "oldFocus"
QT_MOC_LITERAL(6, 64, 20), // "keyboardFocusChanged"
QT_MOC_LITERAL(7, 85, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(8, 102, 20), // "cursorSurfaceRequest"
QT_MOC_LITERAL(9, 123, 7), // "surface"
QT_MOC_LITERAL(10, 131, 8), // "hotspotX"
QT_MOC_LITERAL(11, 140, 8), // "hotspotY"
QT_MOC_LITERAL(12, 149, 12), // "sendKeyEvent"
QT_MOC_LITERAL(13, 162, 5), // "qtKey"
QT_MOC_LITERAL(14, 168, 7), // "pressed"
QT_MOC_LITERAL(15, 176, 21), // "sendTouchPointPressed"
QT_MOC_LITERAL(16, 198, 2), // "id"
QT_MOC_LITERAL(17, 201, 8), // "position"
QT_MOC_LITERAL(18, 210, 22), // "sendTouchPointReleased"
QT_MOC_LITERAL(19, 233, 19), // "sendTouchPointMoved"
QT_MOC_LITERAL(20, 253, 19), // "sendTouchFrameEvent"
QT_MOC_LITERAL(21, 273, 15), // "QWaylandClient*"
QT_MOC_LITERAL(22, 289, 6), // "client"
QT_MOC_LITERAL(23, 296, 20), // "sendTouchCancelEvent"
QT_MOC_LITERAL(24, 317, 4), // "drag"
QT_MOC_LITERAL(25, 322, 13), // "QWaylandDrag*"
QT_MOC_LITERAL(26, 336, 6), // "keymap"
QT_MOC_LITERAL(27, 343, 15), // "QWaylandKeymap*"
QT_MOC_LITERAL(28, 359, 15), // "CapabilityFlags"
QT_MOC_LITERAL(29, 375, 14), // "CapabilityFlag"
QT_MOC_LITERAL(30, 390, 7), // "Pointer"
QT_MOC_LITERAL(31, 398, 8), // "Keyboard"
QT_MOC_LITERAL(32, 407, 5), // "Touch"
QT_MOC_LITERAL(33, 413, 19) // "DefaultCapabilities"

    },
    "QWaylandSeat\0mouseFocusChanged\0\0"
    "QWaylandView*\0newFocus\0oldFocus\0"
    "keyboardFocusChanged\0QWaylandSurface*\0"
    "cursorSurfaceRequest\0surface\0hotspotX\0"
    "hotspotY\0sendKeyEvent\0qtKey\0pressed\0"
    "sendTouchPointPressed\0id\0position\0"
    "sendTouchPointReleased\0sendTouchPointMoved\0"
    "sendTouchFrameEvent\0QWaylandClient*\0"
    "client\0sendTouchCancelEvent\0drag\0"
    "QWaylandDrag*\0keymap\0QWaylandKeymap*\0"
    "CapabilityFlags\0CapabilityFlag\0Pointer\0"
    "Keyboard\0Touch\0DefaultCapabilities"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandSeat[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       2,  108, // properties
       1,  114, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   59,    2, 0x06 /* Public */,
       6,    2,   64,    2, 0x06 /* Public */,
       8,    3,   69,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      12,    2,   76,    2, 0x02 /* Public */,
      15,    3,   81,    2, 0x02 /* Public */,
      18,    3,   88,    2, 0x02 /* Public */,
      19,    3,   95,    2, 0x02 /* Public */,
      20,    1,  102,    2, 0x02 /* Public */,
      23,    1,  105,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    4,    5,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 7,    4,    5,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Int, QMetaType::Int,    9,   10,   11,

 // methods: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,   13,   14,
    QMetaType::UInt, 0x80000000 | 7, QMetaType::Int, QMetaType::QPointF,    9,   16,   17,
    QMetaType::UInt, 0x80000000 | 7, QMetaType::Int, QMetaType::QPointF,    9,   16,   17,
    QMetaType::UInt, 0x80000000 | 7, QMetaType::Int, QMetaType::QPointF,    9,   16,   17,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, 0x80000000 | 21,   22,

 // properties: name, type, flags
      24, 0x80000000 | 25, 0x00095409,
      26, 0x80000000 | 27, 0x00095409,

 // enums: name, alias, flags, count, data
      28,   29, 0x0,    4,  119,

 // enum data: key, value
      30, uint(QWaylandSeat::Pointer),
      31, uint(QWaylandSeat::Keyboard),
      32, uint(QWaylandSeat::Touch),
      33, uint(QWaylandSeat::DefaultCapabilities),

       0        // eod
};

void QWaylandSeat::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandSeat *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->mouseFocusChanged((*reinterpret_cast< QWaylandView*(*)>(_a[1])),(*reinterpret_cast< QWaylandView*(*)>(_a[2]))); break;
        case 1: _t->keyboardFocusChanged((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2]))); break;
        case 2: _t->cursorSurfaceRequest((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 3: _t->sendKeyEvent((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 4: { uint _r = _t->sendTouchPointPressed((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QPointF(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 5: { uint _r = _t->sendTouchPointReleased((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QPointF(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 6: { uint _r = _t->sendTouchPointMoved((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QPointF(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->sendTouchFrameEvent((*reinterpret_cast< QWaylandClient*(*)>(_a[1]))); break;
        case 8: _t->sendTouchCancelEvent((*reinterpret_cast< QWaylandClient*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandClient* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandClient* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandSeat::*)(QWaylandView * , QWaylandView * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSeat::mouseFocusChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandSeat::*)(QWaylandSurface * , QWaylandSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSeat::keyboardFocusChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandSeat::*)(QWaylandSurface * , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSeat::cursorSurfaceRequest)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandSeat *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QWaylandDrag**>(_v) = _t->drag(); break;
        case 1: *reinterpret_cast< QWaylandKeymap**>(_v) = _t->keymap(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandSeat::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandObject::staticMetaObject>(),
    qt_meta_stringdata_QWaylandSeat.data,
    qt_meta_data_QWaylandSeat,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandSeat::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandSeat::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandSeat.stringdata0))
        return static_cast<void*>(this);
    return QWaylandObject::qt_metacast(_clname);
}

int QWaylandSeat::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandSeat::mouseFocusChanged(QWaylandView * _t1, QWaylandView * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QWaylandSeat::keyboardFocusChanged(QWaylandSurface * _t1, QWaylandSurface * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QWaylandSeat::cursorSurfaceRequest(QWaylandSurface * _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
