/* Generated by wayland-scanner 1.21.0 */

#ifndef QT_TEXTURE_SHARING_UNSTABLE_V1_CLIENT_PROTOCOL_H
#define QT_TEXTURE_SHARING_UNSTABLE_V1_CLIENT_PROTOCOL_H

#include <stdint.h>
#include <stddef.h>
#include "wayland-client-core.h"

#ifdef  __cplusplus
extern "C" {
#endif

/**
 * @page page_qt_texture_sharing_unstable_v1 The qt_texture_sharing_unstable_v1 protocol
 * @section page_ifaces_qt_texture_sharing_unstable_v1 Interfaces
 * - @subpage page_iface_zqt_texture_sharing_v1 - 
 * @section page_copyright_qt_texture_sharing_unstable_v1 Copyright
 * <pre>
 *
 * Copyright (C) 2019 The Qt Company Ltd.
 * Contact: http://www.qt.io/licensing/
 *
 * This file is part of the plugins of the Qt Toolkit.
 *
 * $QT_BEGIN_LICENSE:BSD$
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * * Neither the name of The Qt Company Ltd nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 * $QT_END_LICENSE$
 * </pre>
 */
struct qt_server_buffer;
struct zqt_texture_sharing_v1;

#ifndef ZQT_TEXTURE_SHARING_V1_INTERFACE
#define ZQT_TEXTURE_SHARING_V1_INTERFACE
/**
 * @page page_iface_zqt_texture_sharing_v1 zqt_texture_sharing_v1
 * @section page_iface_zqt_texture_sharing_v1_api API
 * See @ref iface_zqt_texture_sharing_v1.
 */
/**
 * @defgroup iface_zqt_texture_sharing_v1 The zqt_texture_sharing_v1 interface
 */
extern const struct wl_interface zqt_texture_sharing_v1_interface;
#endif

/**
 * @ingroup iface_zqt_texture_sharing_v1
 * @struct zqt_texture_sharing_v1_listener
 */
struct zqt_texture_sharing_v1_listener {
	/**
	 */
	void (*image_failed)(void *data,
			     struct zqt_texture_sharing_v1 *zqt_texture_sharing_v1,
			     const char *key,
			     const char *error_message);
	/**
	 */
	void (*provide_buffer)(void *data,
			       struct zqt_texture_sharing_v1 *zqt_texture_sharing_v1,
			       struct qt_server_buffer *buffer,
			       const char *key);
};

/**
 * @ingroup iface_zqt_texture_sharing_v1
 */
static inline int
zqt_texture_sharing_v1_add_listener(struct zqt_texture_sharing_v1 *zqt_texture_sharing_v1,
				    const struct zqt_texture_sharing_v1_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) zqt_texture_sharing_v1,
				     (void (**)(void)) listener, data);
}

#define ZQT_TEXTURE_SHARING_V1_REQUEST_IMAGE 0
#define ZQT_TEXTURE_SHARING_V1_ABANDON_IMAGE 1

/**
 * @ingroup iface_zqt_texture_sharing_v1
 */
#define ZQT_TEXTURE_SHARING_V1_IMAGE_FAILED_SINCE_VERSION 1
/**
 * @ingroup iface_zqt_texture_sharing_v1
 */
#define ZQT_TEXTURE_SHARING_V1_PROVIDE_BUFFER_SINCE_VERSION 1

/**
 * @ingroup iface_zqt_texture_sharing_v1
 */
#define ZQT_TEXTURE_SHARING_V1_REQUEST_IMAGE_SINCE_VERSION 1
/**
 * @ingroup iface_zqt_texture_sharing_v1
 */
#define ZQT_TEXTURE_SHARING_V1_ABANDON_IMAGE_SINCE_VERSION 1

/** @ingroup iface_zqt_texture_sharing_v1 */
static inline void
zqt_texture_sharing_v1_set_user_data(struct zqt_texture_sharing_v1 *zqt_texture_sharing_v1, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) zqt_texture_sharing_v1, user_data);
}

/** @ingroup iface_zqt_texture_sharing_v1 */
static inline void *
zqt_texture_sharing_v1_get_user_data(struct zqt_texture_sharing_v1 *zqt_texture_sharing_v1)
{
	return wl_proxy_get_user_data((struct wl_proxy *) zqt_texture_sharing_v1);
}

static inline uint32_t
zqt_texture_sharing_v1_get_version(struct zqt_texture_sharing_v1 *zqt_texture_sharing_v1)
{
	return wl_proxy_get_version((struct wl_proxy *) zqt_texture_sharing_v1);
}

/** @ingroup iface_zqt_texture_sharing_v1 */
static inline void
zqt_texture_sharing_v1_destroy(struct zqt_texture_sharing_v1 *zqt_texture_sharing_v1)
{
	wl_proxy_destroy((struct wl_proxy *) zqt_texture_sharing_v1);
}

/**
 * @ingroup iface_zqt_texture_sharing_v1
 */
static inline void
zqt_texture_sharing_v1_request_image(struct zqt_texture_sharing_v1 *zqt_texture_sharing_v1, const char *key)
{
	wl_proxy_marshal_flags((struct wl_proxy *) zqt_texture_sharing_v1,
			 ZQT_TEXTURE_SHARING_V1_REQUEST_IMAGE, NULL, wl_proxy_get_version((struct wl_proxy *) zqt_texture_sharing_v1), 0, key);
}

/**
 * @ingroup iface_zqt_texture_sharing_v1
 */
static inline void
zqt_texture_sharing_v1_abandon_image(struct zqt_texture_sharing_v1 *zqt_texture_sharing_v1, const char *key)
{
	wl_proxy_marshal_flags((struct wl_proxy *) zqt_texture_sharing_v1,
			 ZQT_TEXTURE_SHARING_V1_ABANDON_IMAGE, NULL, wl_proxy_get_version((struct wl_proxy *) zqt_texture_sharing_v1), 0, key);
}

#ifdef  __cplusplus
}
#endif

#endif
