/****************************************************************************
** Meta object code from reading C++ file 'qwaylandsurface.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/compositor_api/qwaylandsurface.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandsurface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandSurface_t {
    QByteArrayData data[61];
    char stringdata0[828];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandSurface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandSurface_t qt_meta_stringdata_QWaylandSurface = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QWaylandSurface"
QT_MOC_LITERAL(1, 16, 17), // "hasContentChanged"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 7), // "damaged"
QT_MOC_LITERAL(4, 43, 4), // "rect"
QT_MOC_LITERAL(5, 48, 13), // "parentChanged"
QT_MOC_LITERAL(6, 62, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(7, 79, 9), // "newParent"
QT_MOC_LITERAL(8, 89, 9), // "oldParent"
QT_MOC_LITERAL(9, 99, 10), // "childAdded"
QT_MOC_LITERAL(10, 110, 5), // "child"
QT_MOC_LITERAL(11, 116, 21), // "sourceGeometryChanged"
QT_MOC_LITERAL(12, 138, 22), // "destinationSizeChanged"
QT_MOC_LITERAL(13, 161, 11), // "sizeChanged"
QT_MOC_LITERAL(14, 173, 17), // "bufferSizeChanged"
QT_MOC_LITERAL(15, 191, 18), // "bufferScaleChanged"
QT_MOC_LITERAL(16, 210, 18), // "offsetForNextFrame"
QT_MOC_LITERAL(17, 229, 6), // "offset"
QT_MOC_LITERAL(18, 236, 25), // "contentOrientationChanged"
QT_MOC_LITERAL(19, 262, 16), // "surfaceDestroyed"
QT_MOC_LITERAL(20, 279, 13), // "originChanged"
QT_MOC_LITERAL(21, 293, 25), // "subsurfacePositionChanged"
QT_MOC_LITERAL(22, 319, 8), // "position"
QT_MOC_LITERAL(23, 328, 20), // "subsurfacePlaceAbove"
QT_MOC_LITERAL(24, 349, 7), // "sibling"
QT_MOC_LITERAL(25, 357, 20), // "subsurfacePlaceBelow"
QT_MOC_LITERAL(26, 378, 11), // "dragStarted"
QT_MOC_LITERAL(27, 390, 13), // "QWaylandDrag*"
QT_MOC_LITERAL(28, 404, 4), // "drag"
QT_MOC_LITERAL(29, 409, 20), // "cursorSurfaceChanged"
QT_MOC_LITERAL(30, 430, 19), // "inhibitsIdleChanged"
QT_MOC_LITERAL(31, 450, 9), // "configure"
QT_MOC_LITERAL(32, 460, 9), // "hasBuffer"
QT_MOC_LITERAL(33, 470, 6), // "redraw"
QT_MOC_LITERAL(34, 477, 15), // "updateSelection"
QT_MOC_LITERAL(35, 493, 10), // "initialize"
QT_MOC_LITERAL(36, 504, 19), // "QWaylandCompositor*"
QT_MOC_LITERAL(37, 524, 10), // "compositor"
QT_MOC_LITERAL(38, 535, 15), // "QWaylandClient*"
QT_MOC_LITERAL(39, 551, 6), // "client"
QT_MOC_LITERAL(40, 558, 2), // "id"
QT_MOC_LITERAL(41, 561, 7), // "version"
QT_MOC_LITERAL(42, 569, 7), // "destroy"
QT_MOC_LITERAL(43, 577, 11), // "isDestroyed"
QT_MOC_LITERAL(44, 589, 12), // "frameStarted"
QT_MOC_LITERAL(45, 602, 18), // "sendFrameCallbacks"
QT_MOC_LITERAL(46, 621, 14), // "sourceGeometry"
QT_MOC_LITERAL(47, 636, 15), // "destinationSize"
QT_MOC_LITERAL(48, 652, 10), // "bufferSize"
QT_MOC_LITERAL(49, 663, 4), // "size"
QT_MOC_LITERAL(50, 668, 11), // "bufferScale"
QT_MOC_LITERAL(51, 680, 18), // "contentOrientation"
QT_MOC_LITERAL(52, 699, 21), // "Qt::ScreenOrientation"
QT_MOC_LITERAL(53, 721, 6), // "origin"
QT_MOC_LITERAL(54, 728, 23), // "QWaylandSurface::Origin"
QT_MOC_LITERAL(55, 752, 10), // "hasContent"
QT_MOC_LITERAL(56, 763, 13), // "cursorSurface"
QT_MOC_LITERAL(57, 777, 12), // "inhibitsIdle"
QT_MOC_LITERAL(58, 790, 6), // "Origin"
QT_MOC_LITERAL(59, 797, 13), // "OriginTopLeft"
QT_MOC_LITERAL(60, 811, 16) // "OriginBottomLeft"

    },
    "QWaylandSurface\0hasContentChanged\0\0"
    "damaged\0rect\0parentChanged\0QWaylandSurface*\0"
    "newParent\0oldParent\0childAdded\0child\0"
    "sourceGeometryChanged\0destinationSizeChanged\0"
    "sizeChanged\0bufferSizeChanged\0"
    "bufferScaleChanged\0offsetForNextFrame\0"
    "offset\0contentOrientationChanged\0"
    "surfaceDestroyed\0originChanged\0"
    "subsurfacePositionChanged\0position\0"
    "subsurfacePlaceAbove\0sibling\0"
    "subsurfacePlaceBelow\0dragStarted\0"
    "QWaylandDrag*\0drag\0cursorSurfaceChanged\0"
    "inhibitsIdleChanged\0configure\0hasBuffer\0"
    "redraw\0updateSelection\0initialize\0"
    "QWaylandCompositor*\0compositor\0"
    "QWaylandClient*\0client\0id\0version\0"
    "destroy\0isDestroyed\0frameStarted\0"
    "sendFrameCallbacks\0sourceGeometry\0"
    "destinationSize\0bufferSize\0size\0"
    "bufferScale\0contentOrientation\0"
    "Qt::ScreenOrientation\0origin\0"
    "QWaylandSurface::Origin\0hasContent\0"
    "cursorSurface\0inhibitsIdle\0Origin\0"
    "OriginTopLeft\0OriginBottomLeft"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandSurface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      27,   14, // methods
      11,  231, // properties
       1,  286, // enums/sets
       0,    0, // constructors
       0,       // flags
      21,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  176,    2, 0x06 /* Public */,
       3,    1,  177,    2, 0x06 /* Public */,
       5,    2,  180,    2, 0x06 /* Public */,
       9,    1,  185,    2, 0x06 /* Public */,
      11,    0,  188,    2, 0x86 /* Public | MethodRevisioned */,
      12,    0,  189,    2, 0x86 /* Public | MethodRevisioned */,
      13,    0,  190,    2, 0x06 /* Public */,
      14,    0,  191,    2, 0x86 /* Public | MethodRevisioned */,
      15,    0,  192,    2, 0x06 /* Public */,
      16,    1,  193,    2, 0x06 /* Public */,
      18,    0,  196,    2, 0x06 /* Public */,
      19,    0,  197,    2, 0x06 /* Public */,
      20,    0,  198,    2, 0x06 /* Public */,
      21,    1,  199,    2, 0x06 /* Public */,
      23,    1,  202,    2, 0x06 /* Public */,
      25,    1,  205,    2, 0x06 /* Public */,
      26,    1,  208,    2, 0x06 /* Public */,
      29,    0,  211,    2, 0x06 /* Public */,
      30,    0,  212,    2, 0x86 /* Public | MethodRevisioned */,
      31,    1,  213,    2, 0x06 /* Public */,
      33,    0,  216,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      34,    0,  217,    2, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
      35,    4,  218,    2, 0x02 /* Public */,
      42,    0,  227,    2, 0x02 /* Public */,
      43,    0,  228,    2, 0x02 /* Public */,
      44,    0,  229,    2, 0x02 /* Public */,
      45,    0,  230,    2, 0x02 /* Public */,

 // signals: revision
       0,
       0,
       0,
       0,
      13,
      13,
       0,
      13,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      14,
       0,
       0,

 // slots: revision
       0,

 // methods: revision
       0,
       0,
       0,
       0,
       0,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QRegion,    4,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 6,    7,    8,
    QMetaType::Void, 0x80000000 | 6,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   22,
    QMetaType::Void, 0x80000000 | 6,   24,
    QMetaType::Void, 0x80000000 | 6,   24,
    QMetaType::Void, 0x80000000 | 27,   28,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   32,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 36, 0x80000000 | 38, QMetaType::UInt, QMetaType::Int,   37,   39,   40,   41,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      39, 0x80000000 | 38, 0x00095409,
      46, QMetaType::QRectF, 0x00c95001,
      47, QMetaType::QSize, 0x00c95001,
      48, QMetaType::QSize, 0x00c95001,
      49, QMetaType::QSize, 0x00495001,
      50, QMetaType::Int, 0x00495001,
      51, 0x80000000 | 52, 0x00495009,
      53, 0x80000000 | 54, 0x00495009,
      55, QMetaType::Bool, 0x00495001,
      56, QMetaType::Bool, 0x00495003,
      57, QMetaType::Bool, 0x00c95001,

 // properties: notify_signal_id
       0,
       4,
       5,
       7,
       6,
       8,
      10,
      12,
       0,
      17,
      18,

 // properties: revision
       0,
      13,
      13,
      13,
       0,
       0,
       0,
       0,
       0,
       0,
      14,

 // enums: name, alias, flags, count, data
      58,   58, 0x0,    2,  291,

 // enum data: key, value
      59, uint(QWaylandSurface::OriginTopLeft),
      60, uint(QWaylandSurface::OriginBottomLeft),

       0        // eod
};

void QWaylandSurface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandSurface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->hasContentChanged(); break;
        case 1: _t->damaged((*reinterpret_cast< const QRegion(*)>(_a[1]))); break;
        case 2: _t->parentChanged((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2]))); break;
        case 3: _t->childAdded((*reinterpret_cast< QWaylandSurface*(*)>(_a[1]))); break;
        case 4: _t->sourceGeometryChanged(); break;
        case 5: _t->destinationSizeChanged(); break;
        case 6: _t->sizeChanged(); break;
        case 7: _t->bufferSizeChanged(); break;
        case 8: _t->bufferScaleChanged(); break;
        case 9: _t->offsetForNextFrame((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 10: _t->contentOrientationChanged(); break;
        case 11: _t->surfaceDestroyed(); break;
        case 12: _t->originChanged(); break;
        case 13: _t->subsurfacePositionChanged((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 14: _t->subsurfacePlaceAbove((*reinterpret_cast< QWaylandSurface*(*)>(_a[1]))); break;
        case 15: _t->subsurfacePlaceBelow((*reinterpret_cast< QWaylandSurface*(*)>(_a[1]))); break;
        case 16: _t->dragStarted((*reinterpret_cast< QWaylandDrag*(*)>(_a[1]))); break;
        case 17: _t->cursorSurfaceChanged(); break;
        case 18: _t->inhibitsIdleChanged(); break;
        case 19: _t->configure((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->redraw(); break;
        case 21: _t->updateSelection(); break;
        case 22: _t->initialize((*reinterpret_cast< QWaylandCompositor*(*)>(_a[1])),(*reinterpret_cast< QWaylandClient*(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 23: _t->destroy(); break;
        case 24: { bool _r = _t->isDestroyed();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->frameStarted(); break;
        case 26: _t->sendFrameCallbacks(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandClient* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::hasContentChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)(const QRegion & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::damaged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)(QWaylandSurface * , QWaylandSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::parentChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)(QWaylandSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::childAdded)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::sourceGeometryChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::destinationSizeChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::sizeChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::bufferSizeChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::bufferScaleChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)(const QPoint & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::offsetForNextFrame)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::contentOrientationChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::surfaceDestroyed)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::originChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)(const QPoint & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::subsurfacePositionChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)(QWaylandSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::subsurfacePlaceAbove)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)(QWaylandSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::subsurfacePlaceBelow)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)(QWaylandDrag * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::dragStarted)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::cursorSurfaceChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::inhibitsIdleChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::configure)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QWaylandSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandSurface::redraw)) {
                *result = 20;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandClient* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandSurface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QWaylandClient**>(_v) = _t->client(); break;
        case 1: *reinterpret_cast< QRectF*>(_v) = _t->sourceGeometry(); break;
        case 2: *reinterpret_cast< QSize*>(_v) = _t->destinationSize(); break;
        case 3: *reinterpret_cast< QSize*>(_v) = _t->bufferSize(); break;
        case 4: *reinterpret_cast< QSize*>(_v) = _t->size(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->bufferScale(); break;
        case 6: *reinterpret_cast< Qt::ScreenOrientation*>(_v) = _t->contentOrientation(); break;
        case 7: *reinterpret_cast< QWaylandSurface::Origin*>(_v) = _t->origin(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->hasContent(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->isCursorSurface(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->inhibitsIdle(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QWaylandSurface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 9: _t->markAsCursorSurface(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandSurface::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandObject::staticMetaObject>(),
    qt_meta_stringdata_QWaylandSurface.data,
    qt_meta_data_QWaylandSurface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandSurface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandSurface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandSurface.stringdata0))
        return static_cast<void*>(this);
    return QWaylandObject::qt_metacast(_clname);
}

int QWaylandSurface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 11;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandSurface::hasContentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandSurface::damaged(const QRegion & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QWaylandSurface::parentChanged(QWaylandSurface * _t1, QWaylandSurface * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QWaylandSurface::childAdded(QWaylandSurface * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QWaylandSurface::sourceGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QWaylandSurface::destinationSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QWaylandSurface::sizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QWaylandSurface::bufferSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QWaylandSurface::bufferScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QWaylandSurface::offsetForNextFrame(const QPoint & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QWaylandSurface::contentOrientationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QWaylandSurface::surfaceDestroyed()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QWaylandSurface::originChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QWaylandSurface::subsurfacePositionChanged(const QPoint & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QWaylandSurface::subsurfacePlaceAbove(QWaylandSurface * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QWaylandSurface::subsurfacePlaceBelow(QWaylandSurface * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QWaylandSurface::dragStarted(QWaylandDrag * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QWaylandSurface::cursorSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QWaylandSurface::inhibitsIdleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void QWaylandSurface::configure(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void QWaylandSurface::redraw()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
