/****************************************************************************
** Meta object code from reading C++ file 'qwaylandquickitem.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/compositor_api/qwaylandquickitem.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandquickitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandQuickItem_t {
    QByteArrayData data[75];
    char stringdata0[1115];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandQuickItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandQuickItem_t qt_meta_stringdata_QWaylandQuickItem = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QWaylandQuickItem"
QT_MOC_LITERAL(1, 18, 14), // "surfaceChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 17), // "compositorChanged"
QT_MOC_LITERAL(4, 52, 19), // "paintEnabledChanged"
QT_MOC_LITERAL(5, 72, 25), // "touchEventsEnabledChanged"
QT_MOC_LITERAL(6, 98, 13), // "originChanged"
QT_MOC_LITERAL(7, 112, 16), // "surfaceDestroyed"
QT_MOC_LITERAL(8, 129, 25), // "inputEventsEnabledChanged"
QT_MOC_LITERAL(9, 155, 19), // "focusOnClickChanged"
QT_MOC_LITERAL(10, 175, 9), // "mouseMove"
QT_MOC_LITERAL(11, 185, 14), // "windowPosition"
QT_MOC_LITERAL(12, 200, 12), // "mouseRelease"
QT_MOC_LITERAL(13, 213, 25), // "sizeFollowsSurfaceChanged"
QT_MOC_LITERAL(14, 239, 24), // "subsurfaceHandlerChanged"
QT_MOC_LITERAL(15, 264, 13), // "outputChanged"
QT_MOC_LITERAL(16, 278, 19), // "bufferLockedChanged"
QT_MOC_LITERAL(17, 298, 30), // "allowDiscardFrontBufferChanged"
QT_MOC_LITERAL(18, 329, 9), // "takeFocus"
QT_MOC_LITERAL(19, 339, 13), // "QWaylandSeat*"
QT_MOC_LITERAL(20, 353, 6), // "device"
QT_MOC_LITERAL(21, 360, 15), // "setPaintEnabled"
QT_MOC_LITERAL(22, 376, 12), // "paintEnabled"
QT_MOC_LITERAL(23, 389, 5), // "raise"
QT_MOC_LITERAL(24, 395, 5), // "lower"
QT_MOC_LITERAL(25, 401, 18), // "sendMouseMoveEvent"
QT_MOC_LITERAL(26, 420, 8), // "position"
QT_MOC_LITERAL(27, 429, 4), // "seat"
QT_MOC_LITERAL(28, 434, 20), // "surfaceMappedChanged"
QT_MOC_LITERAL(29, 455, 20), // "handleSurfaceChanged"
QT_MOC_LITERAL(30, 476, 13), // "parentChanged"
QT_MOC_LITERAL(31, 490, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(32, 507, 9), // "newParent"
QT_MOC_LITERAL(33, 517, 9), // "oldParent"
QT_MOC_LITERAL(34, 527, 10), // "updateSize"
QT_MOC_LITERAL(35, 538, 12), // "updateBuffer"
QT_MOC_LITERAL(36, 551, 9), // "hasBuffer"
QT_MOC_LITERAL(37, 561, 12), // "updateWindow"
QT_MOC_LITERAL(38, 574, 12), // "updateOutput"
QT_MOC_LITERAL(39, 587, 10), // "beforeSync"
QT_MOC_LITERAL(40, 598, 21), // "handleSubsurfaceAdded"
QT_MOC_LITERAL(41, 620, 12), // "childSurface"
QT_MOC_LITERAL(42, 633, 24), // "handleSubsurfacePosition"
QT_MOC_LITERAL(43, 658, 3), // "pos"
QT_MOC_LITERAL(44, 662, 16), // "handlePlaceAbove"
QT_MOC_LITERAL(45, 679, 16), // "referenceSurface"
QT_MOC_LITERAL(46, 696, 16), // "handlePlaceBelow"
QT_MOC_LITERAL(47, 713, 17), // "handleDragStarted"
QT_MOC_LITERAL(48, 731, 13), // "QWaylandDrag*"
QT_MOC_LITERAL(49, 745, 4), // "drag"
QT_MOC_LITERAL(50, 750, 17), // "updateInputMethod"
QT_MOC_LITERAL(51, 768, 22), // "Qt::InputMethodQueries"
QT_MOC_LITERAL(52, 791, 7), // "queries"
QT_MOC_LITERAL(53, 799, 12), // "mapToSurface"
QT_MOC_LITERAL(54, 812, 5), // "point"
QT_MOC_LITERAL(55, 818, 14), // "mapFromSurface"
QT_MOC_LITERAL(56, 833, 16), // "inputMethodQuery"
QT_MOC_LITERAL(57, 850, 20), // "Qt::InputMethodQuery"
QT_MOC_LITERAL(58, 871, 5), // "query"
QT_MOC_LITERAL(59, 877, 8), // "argument"
QT_MOC_LITERAL(60, 886, 10), // "setPrimary"
QT_MOC_LITERAL(61, 897, 10), // "compositor"
QT_MOC_LITERAL(62, 908, 19), // "QWaylandCompositor*"
QT_MOC_LITERAL(63, 928, 7), // "surface"
QT_MOC_LITERAL(64, 936, 18), // "touchEventsEnabled"
QT_MOC_LITERAL(65, 955, 6), // "origin"
QT_MOC_LITERAL(66, 962, 23), // "QWaylandSurface::Origin"
QT_MOC_LITERAL(67, 986, 18), // "inputEventsEnabled"
QT_MOC_LITERAL(68, 1005, 12), // "focusOnClick"
QT_MOC_LITERAL(69, 1018, 18), // "sizeFollowsSurface"
QT_MOC_LITERAL(70, 1037, 17), // "subsurfaceHandler"
QT_MOC_LITERAL(71, 1055, 6), // "output"
QT_MOC_LITERAL(72, 1062, 15), // "QWaylandOutput*"
QT_MOC_LITERAL(73, 1078, 12), // "bufferLocked"
QT_MOC_LITERAL(74, 1091, 23) // "allowDiscardFrontBuffer"

    },
    "QWaylandQuickItem\0surfaceChanged\0\0"
    "compositorChanged\0paintEnabledChanged\0"
    "touchEventsEnabledChanged\0originChanged\0"
    "surfaceDestroyed\0inputEventsEnabledChanged\0"
    "focusOnClickChanged\0mouseMove\0"
    "windowPosition\0mouseRelease\0"
    "sizeFollowsSurfaceChanged\0"
    "subsurfaceHandlerChanged\0outputChanged\0"
    "bufferLockedChanged\0allowDiscardFrontBufferChanged\0"
    "takeFocus\0QWaylandSeat*\0device\0"
    "setPaintEnabled\0paintEnabled\0raise\0"
    "lower\0sendMouseMoveEvent\0position\0"
    "seat\0surfaceMappedChanged\0"
    "handleSurfaceChanged\0parentChanged\0"
    "QWaylandSurface*\0newParent\0oldParent\0"
    "updateSize\0updateBuffer\0hasBuffer\0"
    "updateWindow\0updateOutput\0beforeSync\0"
    "handleSubsurfaceAdded\0childSurface\0"
    "handleSubsurfacePosition\0pos\0"
    "handlePlaceAbove\0referenceSurface\0"
    "handlePlaceBelow\0handleDragStarted\0"
    "QWaylandDrag*\0drag\0updateInputMethod\0"
    "Qt::InputMethodQueries\0queries\0"
    "mapToSurface\0point\0mapFromSurface\0"
    "inputMethodQuery\0Qt::InputMethodQuery\0"
    "query\0argument\0setPrimary\0compositor\0"
    "QWaylandCompositor*\0surface\0"
    "touchEventsEnabled\0origin\0"
    "QWaylandSurface::Origin\0inputEventsEnabled\0"
    "focusOnClick\0sizeFollowsSurface\0"
    "subsurfaceHandler\0output\0QWaylandOutput*\0"
    "bufferLocked\0allowDiscardFrontBuffer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandQuickItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      40,   14, // methods
      12,  332, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      15,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  254,    2, 0x06 /* Public */,
       3,    0,  255,    2, 0x06 /* Public */,
       4,    0,  256,    2, 0x06 /* Public */,
       5,    0,  257,    2, 0x06 /* Public */,
       6,    0,  258,    2, 0x06 /* Public */,
       7,    0,  259,    2, 0x06 /* Public */,
       8,    0,  260,    2, 0x06 /* Public */,
       9,    0,  261,    2, 0x06 /* Public */,
      10,    1,  262,    2, 0x06 /* Public */,
      12,    0,  265,    2, 0x06 /* Public */,
      13,    0,  266,    2, 0x06 /* Public */,
      14,    0,  267,    2, 0x06 /* Public */,
      15,    0,  268,    2, 0x06 /* Public */,
      16,    0,  269,    2, 0x06 /* Public */,
      17,    0,  270,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    1,  271,    2, 0x0a /* Public */,
      18,    0,  274,    2, 0x2a /* Public | MethodCloned */,
      21,    1,  275,    2, 0x0a /* Public */,
      23,    0,  278,    2, 0x0a /* Public */,
      24,    0,  279,    2, 0x0a /* Public */,
      25,    2,  280,    2, 0x0a /* Public */,
      25,    1,  285,    2, 0x2a /* Public | MethodCloned */,
      28,    0,  288,    2, 0x08 /* Private */,
      29,    0,  289,    2, 0x08 /* Private */,
      30,    2,  290,    2, 0x08 /* Private */,
      34,    0,  295,    2, 0x08 /* Private */,
      35,    1,  296,    2, 0x08 /* Private */,
      37,    0,  299,    2, 0x08 /* Private */,
      38,    0,  300,    2, 0x08 /* Private */,
      39,    0,  301,    2, 0x08 /* Private */,
      40,    1,  302,    2, 0x08 /* Private */,
      42,    1,  305,    2, 0x08 /* Private */,
      44,    1,  308,    2, 0x08 /* Private */,
      46,    1,  311,    2, 0x08 /* Private */,
      47,    1,  314,    2, 0x08 /* Private */,
      50,    1,  317,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      53,    1,  320,    2, 0x02 /* Public */,
      55,    1,  323,    2, 0x82 /* Public | MethodRevisioned */,
      56,    2,  326,    2, 0x02 /* Public */,
      60,    0,  331,    2, 0x02 /* Public */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // slots: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // methods: revision
       0,
      13,
       0,
       0,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPointF,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 19,   20,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPointF, 0x80000000 | 19,   26,   27,
    QMetaType::Void, QMetaType::QPointF,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 31, 0x80000000 | 31,   32,   33,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   36,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 31,   41,
    QMetaType::Void, QMetaType::QPoint,   43,
    QMetaType::Void, 0x80000000 | 31,   45,
    QMetaType::Void, 0x80000000 | 31,   45,
    QMetaType::Void, 0x80000000 | 48,   49,
    QMetaType::Void, 0x80000000 | 51,   52,

 // methods: parameters
    QMetaType::QPointF, QMetaType::QPointF,   54,
    QMetaType::QPointF, QMetaType::QPointF,   54,
    QMetaType::QVariant, 0x80000000 | 57, QMetaType::QVariant,   58,   59,
    QMetaType::Void,

 // properties: name, type, flags
      61, 0x80000000 | 62, 0x00495009,
      63, 0x80000000 | 31, 0x0049510b,
      22, QMetaType::Bool, 0x00495103,
      64, QMetaType::Bool, 0x00495103,
      65, 0x80000000 | 66, 0x00495009,
      67, QMetaType::Bool, 0x00495103,
      68, QMetaType::Bool, 0x00495103,
      69, QMetaType::Bool, 0x00495103,
      70, QMetaType::QObjectStar, 0x00495103,
      71, 0x80000000 | 72, 0x0049510b,
      73, QMetaType::Bool, 0x00495103,
      74, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       1,
       0,
       2,
       3,
       4,
       6,
       7,
      10,
      11,
      12,
      13,
      14,

       0        // eod
};

void QWaylandQuickItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandQuickItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->surfaceChanged(); break;
        case 1: _t->compositorChanged(); break;
        case 2: _t->paintEnabledChanged(); break;
        case 3: _t->touchEventsEnabledChanged(); break;
        case 4: _t->originChanged(); break;
        case 5: _t->surfaceDestroyed(); break;
        case 6: _t->inputEventsEnabledChanged(); break;
        case 7: _t->focusOnClickChanged(); break;
        case 8: _t->mouseMove((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 9: _t->mouseRelease(); break;
        case 10: _t->sizeFollowsSurfaceChanged(); break;
        case 11: _t->subsurfaceHandlerChanged(); break;
        case 12: _t->outputChanged(); break;
        case 13: _t->bufferLockedChanged(); break;
        case 14: _t->allowDiscardFrontBufferChanged(); break;
        case 15: _t->takeFocus((*reinterpret_cast< QWaylandSeat*(*)>(_a[1]))); break;
        case 16: _t->takeFocus(); break;
        case 17: _t->setPaintEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->raise(); break;
        case 19: _t->lower(); break;
        case 20: _t->sendMouseMoveEvent((*reinterpret_cast< const QPointF(*)>(_a[1])),(*reinterpret_cast< QWaylandSeat*(*)>(_a[2]))); break;
        case 21: _t->sendMouseMoveEvent((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 22: _t->surfaceMappedChanged(); break;
        case 23: _t->handleSurfaceChanged(); break;
        case 24: _t->parentChanged((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2]))); break;
        case 25: _t->updateSize(); break;
        case 26: _t->updateBuffer((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 27: _t->updateWindow(); break;
        case 28: _t->updateOutput(); break;
        case 29: _t->beforeSync(); break;
        case 30: _t->handleSubsurfaceAdded((*reinterpret_cast< QWaylandSurface*(*)>(_a[1]))); break;
        case 31: _t->handleSubsurfacePosition((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 32: _t->handlePlaceAbove((*reinterpret_cast< QWaylandSurface*(*)>(_a[1]))); break;
        case 33: _t->handlePlaceBelow((*reinterpret_cast< QWaylandSurface*(*)>(_a[1]))); break;
        case 34: _t->handleDragStarted((*reinterpret_cast< QWaylandDrag*(*)>(_a[1]))); break;
        case 35: _t->updateInputMethod((*reinterpret_cast< Qt::InputMethodQueries(*)>(_a[1]))); break;
        case 36: { QPointF _r = _t->mapToSurface((*reinterpret_cast< const QPointF(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QPointF*>(_a[0]) = std::move(_r); }  break;
        case 37: { QPointF _r = _t->mapFromSurface((*reinterpret_cast< const QPointF(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QPointF*>(_a[0]) = std::move(_r); }  break;
        case 38: { QVariant _r = _t->inputMethodQuery((*reinterpret_cast< Qt::InputMethodQuery(*)>(_a[1])),(*reinterpret_cast< QVariant(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 39: _t->setPrimary(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::surfaceChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::compositorChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::paintEnabledChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::touchEventsEnabledChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::originChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::surfaceDestroyed)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::inputEventsEnabledChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::focusOnClickChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)(const QPointF & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::mouseMove)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::mouseRelease)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::sizeFollowsSurfaceChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::subsurfaceHandlerChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::outputChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::bufferLockedChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickItem::allowDiscardFrontBufferChanged)) {
                *result = 14;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandQuickItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QWaylandCompositor**>(_v) = _t->compositor(); break;
        case 1: *reinterpret_cast< QWaylandSurface**>(_v) = _t->surface(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->paintEnabled(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->touchEventsEnabled(); break;
        case 4: *reinterpret_cast< QWaylandSurface::Origin*>(_v) = _t->origin(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->inputEventsEnabled(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->focusOnClick(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->sizeFollowsSurface(); break;
        case 8: *reinterpret_cast< QObject**>(_v) = _t->subsurfaceHandler(); break;
        case 9: *reinterpret_cast< QWaylandOutput**>(_v) = _t->output(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->isBufferLocked(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->allowDiscardFrontBuffer(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QWaylandQuickItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSurface(*reinterpret_cast< QWaylandSurface**>(_v)); break;
        case 2: _t->setPaintEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setTouchEventsEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setInputEventsEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setFocusOnClick(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setSizeFollowsSurface(*reinterpret_cast< bool*>(_v)); break;
        case 8: _t->setSubsurfaceHandler(*reinterpret_cast< QObject**>(_v)); break;
        case 9: _t->setOutput(*reinterpret_cast< QWaylandOutput**>(_v)); break;
        case 10: _t->setBufferLocked(*reinterpret_cast< bool*>(_v)); break;
        case 11: _t->setAllowDiscardFrontBuffer(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

static const QMetaObject::SuperData qt_meta_extradata_QWaylandQuickItem[] = {
    QMetaObject::SuperData::link<QWaylandSurface::staticMetaObject>(),
    nullptr
};

QT_INIT_METAOBJECT const QMetaObject QWaylandQuickItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QWaylandQuickItem.data,
    qt_meta_data_QWaylandQuickItem,
    qt_static_metacall,
    qt_meta_extradata_QWaylandQuickItem,
    nullptr
} };


const QMetaObject *QWaylandQuickItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandQuickItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandQuickItem.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QWaylandQuickItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 12;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandQuickItem::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandQuickItem::compositorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QWaylandQuickItem::paintEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QWaylandQuickItem::touchEventsEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QWaylandQuickItem::originChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QWaylandQuickItem::surfaceDestroyed()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QWaylandQuickItem::inputEventsEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QWaylandQuickItem::focusOnClickChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QWaylandQuickItem::mouseMove(const QPointF & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QWaylandQuickItem::mouseRelease()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QWaylandQuickItem::sizeFollowsSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QWaylandQuickItem::subsurfaceHandlerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QWaylandQuickItem::outputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QWaylandQuickItem::bufferLockedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QWaylandQuickItem::allowDiscardFrontBufferChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
