/****************************************************************************
** Meta object code from reading C++ file 'qwltexturesharingextension_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/extensions/qwltexturesharingextension_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwltexturesharingextension_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandTextureSharingExtension_t {
    QByteArrayData data[9];
    char stringdata0[127];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandTextureSharingExtension_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandTextureSharingExtension_t qt_meta_stringdata_QWaylandTextureSharingExtension = {
    {
QT_MOC_LITERAL(0, 0, 31), // "QWaylandTextureSharingExtension"
QT_MOC_LITERAL(1, 32, 12), // "bufferResult"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 3), // "key"
QT_MOC_LITERAL(4, 50, 24), // "QtWayland::ServerBuffer*"
QT_MOC_LITERAL(5, 75, 6), // "buffer"
QT_MOC_LITERAL(6, 82, 13), // "requestBuffer"
QT_MOC_LITERAL(7, 96, 14), // "cleanupBuffers"
QT_MOC_LITERAL(8, 111, 15) // "imageSearchPath"

    },
    "QWaylandTextureSharingExtension\0"
    "bufferResult\0\0key\0QtWayland::ServerBuffer*\0"
    "buffer\0requestBuffer\0cleanupBuffers\0"
    "imageSearchPath"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandTextureSharingExtension[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       1,   38, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   34,    2, 0x0a /* Public */,
       7,    0,   37,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, 0x80000000 | 4,    3,    5,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,

 // properties: name, type, flags
       8, QMetaType::QString, 0x00095102,

       0        // eod
};

void QWaylandTextureSharingExtension::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandTextureSharingExtension *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->bufferResult((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QtWayland::ServerBuffer*(*)>(_a[2]))); break;
        case 1: _t->requestBuffer((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->cleanupBuffers(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandTextureSharingExtension::*)(const QString & , QtWayland::ServerBuffer * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandTextureSharingExtension::bufferResult)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QWaylandTextureSharingExtension *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setImageSearchPath(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandTextureSharingExtension::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandCompositorExtensionTemplate<QWaylandTextureSharingExtension>::staticMetaObject>(),
    qt_meta_stringdata_QWaylandTextureSharingExtension.data,
    qt_meta_data_QWaylandTextureSharingExtension,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandTextureSharingExtension::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandTextureSharingExtension::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandTextureSharingExtension.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWaylandServer::zqt_texture_sharing_v1"))
        return static_cast< QtWaylandServer::zqt_texture_sharing_v1*>(this);
    return QWaylandCompositorExtensionTemplate<QWaylandTextureSharingExtension>::qt_metacast(_clname);
}

int QWaylandTextureSharingExtension::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandCompositorExtensionTemplate<QWaylandTextureSharingExtension>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandTextureSharingExtension::bufferResult(const QString & _t1, QtWayland::ServerBuffer * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_QWaylandTextureSharingExtensionQuickExtension_t {
    QByteArrayData data[4];
    char stringdata0[93];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandTextureSharingExtensionQuickExtension_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandTextureSharingExtensionQuickExtension_t qt_meta_stringdata_QWaylandTextureSharingExtensionQuickExtension = {
    {
QT_MOC_LITERAL(0, 0, 45), // "QWaylandTextureSharingExtensi..."
QT_MOC_LITERAL(1, 46, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 62, 4), // "data"
QT_MOC_LITERAL(3, 67, 25) // "QQmlListProperty<QObject>"

    },
    "QWaylandTextureSharingExtensionQuickExtension\0"
    "DefaultProperty\0data\0QQmlListProperty<QObject>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandTextureSharingExtensionQuickExtension[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       1,   16, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // properties: name, type, flags
       2, 0x80000000 | 3, 0x00094009,

       0        // eod
};

void QWaylandTextureSharingExtensionQuickExtension::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QObject> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandTextureSharingExtensionQuickExtension *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->data(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
}

QT_INIT_METAOBJECT const QMetaObject QWaylandTextureSharingExtensionQuickExtension::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandTextureSharingExtension::staticMetaObject>(),
    qt_meta_stringdata_QWaylandTextureSharingExtensionQuickExtension.data,
    qt_meta_data_QWaylandTextureSharingExtensionQuickExtension,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandTextureSharingExtensionQuickExtension::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandTextureSharingExtensionQuickExtension::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandTextureSharingExtensionQuickExtension.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QWaylandTextureSharingExtension::qt_metacast(_clname);
}

int QWaylandTextureSharingExtensionQuickExtension::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandTextureSharingExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
